package org.sample.nio.connection.handler.filter;

import java.io.IOException;
import java.net.SocketAddress;
import java.nio.ByteBuffer;
import java.nio.channels.DatagramChannel;
import java.nio.channels.SelectableChannel;
import java.nio.channels.SelectionKey;
import java.util.logging.Level;

import org.sample.nio.QuoteManager;
import org.sample.nio.connection.handler.ClientConnectionHandler;

import com.sun.grizzly.Context;
import com.sun.grizzly.Controller;
import com.sun.grizzly.ProtocolFilter;
import com.sun.grizzly.ProtocolParser;
import com.sun.grizzly.SelectorHandler;
import com.sun.grizzly.TCPSelectorHandler;
import com.sun.grizzly.Context.AttributeScope;
import com.sun.grizzly.filter.ReadFilter;
import com.sun.grizzly.util.AttributeHolder;
import com.sun.grizzly.util.OutputWriter;
import com.sun.grizzly.util.WorkerThread;

public class QuoteQueryManagerFilter implements ProtocolFilter {
	
	private static final String CLIENT_CONNECTION_HANDLER_ATTR = "connection-handler";
	 
    private QuoteManager manager;

    public QuoteQueryManagerFilter(QuoteManager manager) {
        this.manager = manager;
    }

    public boolean execute(Context context) throws IOException {
        String query = (String) context.removeAttribute(ProtocolParser.MESSAGE);

        if(query==null || query.trim().length()==0){
        	return false;
        }
        
        System.out.println("query = " + query);
        
        
        //here should be the parsed query command string with
        // which query should be able to create a command
        // I have no time to rewrite ClientConnectionHandler
        // Basically you have in context a link to client connection
        //with that  and for example TCPSelectorHandler your commands can 
        //send data to client
        
        /*
         * For now instead of keeping whole Context, you can just keep SelectionKey as reference to the client,
         * and send response back using: selectorHandler.getAsyncQueueWriter().write(SelectionKey, ...);
         */
        
        // on va chercher le ClientConnectionHandler si il existe
        
        ClientConnectionHandler clientConnectionHandler = retrieveConnectionHandler(context);
        
        if(query.equals("MAX")){
        	clientConnectionHandler.close();
        	return false;
        }
        
        manager.processQuery(clientConnectionHandler, query);
        
        
//        // Depending on protocol perform echo
//        //SelectableChannel channel = context.getSelectionKey().channel();
//        try {
//        	
//        	ByteBuffer writeBuffer = ByteBuffer.allocateDirect(query.length());
//    		
//    		writeBuffer.put(query.getBytes());
//    		
//    		writeBuffer.flip();
//        	
//    		SelectionKey key = context.getSelectionKey();
//            
//    		SelectorHandler selectorHandler = context.getSelectorHandler();
//            
//            selectorHandler.getAsyncQueueWriter().write(key,writeBuffer);
//            
//            /*
//    		if (context.getProtocol() == Controller.Protocol.TCP) { // TCP case
//                OutputWriter.flushChannel(channel, writeBuffer);
//            } 
//            */
//    		//context.getAsyncQueueWritable().writeToAsyncQueue(writeBuffer, null, null, true);
//        } catch (IOException ex) {
//            throw ex;
//        }
    
        /*
         * The former will block until all the bytes are written. When I say block, 
         * I mean the calling Thread will block. The OutputWriter internally use a pool of 
         * temporary Selector to make sure writes are always executed entirely.

		With async queue write (the later), the calling thread will not block. instead, 
		the async queue mechanism will make sure your write operation is successful. 
		Depending on the protocol, picking one or the other make a big difference. 
		HTTP use OutputWriter, SIP use async queue.

         */

        return true;
    }

    public boolean postExecute(Context context) throws IOException {
        return true;
    }
    
    private ClientConnectionHandler retrieveConnectionHandler(Context context) {
        ClientConnectionHandler clientConnectionHandler = null;

        AttributeHolder connectionAttrs = context.getAttributeHolderByScope(AttributeScope.CONNECTION);
        if (connectionAttrs != null) {
            clientConnectionHandler = (ClientConnectionHandler) connectionAttrs.getAttribute(CLIENT_CONNECTION_HANDLER_ATTR);
        } else {
            WorkerThread workerThread = (WorkerThread) Thread.currentThread(); // Detach the current Thread data.
            connectionAttrs = workerThread.getAttachment();
            // Attach it to the SelectionKey so the it can be resumed latter.
            context.getSelectionKey().attach(connectionAttrs);
        }
        
        if (clientConnectionHandler == null) {
            clientConnectionHandler = new ClientConnectionHandler(manager, context.getSelectionKey(), context.getSelectorHandler());
            connectionAttrs.setAttribute(CLIENT_CONNECTION_HANDLER_ATTR, clientConnectionHandler);
        }

        return clientConnectionHandler;
    }
    
}

